/*
 * pdwon.c : setup EINT1 for power-down and wakeup
 *   by uratan! 2008.9.9
 */
#include "common.h"
#include "LPC214x.h"
#include "irq.h"
#include "swi.h"
#include "pcon.h"

/*
 * register bitmaps
 */
/* EXTINT: External Interrupt Flag register */
#define EINT0		0x00000001
#define EINT1		0x00000002
#define EINT2		0x00000004
#define EINT3		0x00000008
/* INTWAKE: Interrupt Wakeup register */
#define EXTWAKE0	0x00000001
#define EXTWAKE1	0x00000002
#define EXTWAKE2	0x00000004
#define EXTWAKE3	0x00000008
#define USBWAKE		0x00000020
#define BODWAKE		0x00004000
#define RTCWAKE		0x00008000
/* EXTMODE: External Interrupt Mode register */
#define EINT0_EDGE	0x00000001
#define EINT1_EDGE	0x00000002
#define EINT2_EDGE	0x00000004
#define EINT3_EDGE	0x00000008
/* EXTPOLAR: External Interrupt Polarity register */
#define EINT0_RISING	0x00000001
#define EINT1_RISING	0x00000002
#define EINT2_RISING	0x00000004
#define EINT3_RISING	0x00000008

/*
 * functions
 */
void pdown_init(void);
int power_down(int en_eint);
void pdown_sys_pll_init(void);
void pdown_usb_pll_init(void);

//int once_stop;
//int once_stop48;

/*
 *
 */
void pdown_init(void)
{
	PINSEL0 |= (3<<6);			/* set P0.3 as EINT1 */
//	PINSEL0 |= (2<<28);			/* set P0.14 as EINT1 (for test) */

	INTWAKE = 0;				/* disable wake-up function */
	EXTMODE = EINT1_EDGE;		/* INT1 edge trigger */
	EXTPOLAR = 0;				/* INT1 is falling edge by default */
		/*
		 * no need to use intr-handler to wake up operation
		 */
	return;
}

/*
 *
 */
int power_down(int en_eint)
{
	u_long imf;
	volatile u_long t;
	int wakeup_by_eint;

	imf = IntDisable();				/* disable interrupts */

	/*
	 * once clear request here.
	 *   (means ignore wake-up request before here)
	 */
	EXTINT = EINT1;

	/*
	 * wait for UART job finished (about 1.3msec with -Os, 60MHz)
	 */
	for(t=0; t<5000; t++)
		;
//rprintf("set wake-up request, waiting trigger ");
//cm_getchar();
//rprintf("\r\n");
//rprintf("EXTINT is %04x\r\n", EXTINT);
//for(t=0; t<5000; t++)
//	;

	/*
	 * select wake up source
	 */
	if(en_eint) {
		INTWAKE = USBWAKE | EXTWAKE1;	/* USB or EINT1 can wake up me */
	} else {
		INTWAKE = USBWAKE;				/* only USB can wake up me */
	}
//	INTWAKE = EXTWAKE1;				/* EINT1 can wake up me (for test) */
//	INTWAKE = USBWAKE | EXTWAKE1;	/* USB or EINT1 can wake up me (for test) */

	/*
	 * then try to Power Down
	 *   this system will never enter Power Down mode if:
	 *     1) wake-up request by EINT1 is already exist.
	 *     2) USB bus activity is already recoverd (USB_need_clk=1).
	 *   (in these case, 2 PLLs keep active)
	 */
	PCON = PC_PD;
		/*
		 * I am in power down mode now! ... maybe
		 */

	/*
	 * I have waked up now
	 */
	wakeup_by_eint = (EXTINT & EINT1) ? 1 : 0;	/* get status */
	INTWAKE = 0;				/* disable wake-up function */

	/*
	 * re-initial both PLL if needed
	 */
	if((PLLSTAT & (1<<10)) == 0) {
		pdown_sys_pll_init();
//		once_stop = 1;
	}
	if((PLL48STAT & (1<<10)) == 0) {
		pdown_usb_pll_init();
//		once_stop48 = 1;
	}
//rprintf("once_stop is %d\r\n", once_stop);
//rprintf("once_stop48 is %d\r\n", once_stop48);
//rprintf("EXTINT is %04x\r\n", wakeup_by_eint);
/*
 * clear some status (recv-buff, etc.)
 */

	/*
	 * recover previous state
	 */
	IntRestore(imf);				/* enable interrupts */

	return(wakeup_by_eint);
}

/*
 *
 */
void pdown_sys_pll_init(void)
{
	/*
	 * initial PLL for system clock
	 */
	PLLCON = 1;				/* enable PLL */

#if (Fcco / Fcclk) == 2
	PLLCFG = ((Fcclk / Fosc) - 1) | (0 << 5);
#elif (Fcco / Fcclk) == 4
	PLLCFG = ((Fcclk / Fosc) - 1) | (1 << 5);			/* <== */
#elif (Fcco / Fcclk) == 8
	PLLCFG = ((Fcclk / Fosc) - 1) | (2 << 5);
#elif (Fcco / Fcclk) == 16
	PLLCFG = ((Fcclk / Fosc) - 1) | (3 << 5);
#else
#error invalid PLLCFG
#endif
	PLLFEED = 0xaa;
	PLLFEED = 0x55;

	/*
	 * wait for PLL lock
	 */
	while((PLLSTAT & (1<<10)) == 0)
		;

	PLLCON = 3;				/* enable & connect PLL */
	PLLFEED = 0xaa;
	PLLFEED = 0x55;
}

/*
 * Configure 48MHz USB Clock from Fosc=12MHz
 */
void pdown_usb_pll_init(void)
{
	PLL48CON  = 1;			/* enable PLL */
	PLL48CFG  = 0x23;		/* M = 4, P = 2 */
	PLL48FEED = 0xaa;
	PLL48FEED = 0x55;

	/*
	 * wait for PLL lock
	 */
	while((PLL48STAT & (1<<10)) == 0)
		;

	PLL48CON  = 3;			/* enable & connect PLL */
	PLL48FEED = 0xaa;
	PLL48FEED = 0x55;
}
